"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createContextManager = createContextManager;
const mergeInto_1 = require("../../tools/mergeInto");
const typeUtils_1 = require("../../tools/utils/typeUtils");
const sanitize_1 = require("../../tools/serialisation/sanitize");
const observable_1 = require("../../tools/observable");
const display_1 = require("../../tools/display");
function ensureProperties(context, propertiesConfig, name) {
    const newContext = { ...context };
    for (const [key, { required, type }] of Object.entries(propertiesConfig)) {
        /**
         * Ensure specified properties are strings as defined here:
         * https://docs.datadoghq.com/logs/log_configuration/attributes_naming_convention/#user-related-attributes
         */
        if (type === 'string' && key in newContext) {
            /* eslint-disable @typescript-eslint/no-base-to-string */
            newContext[key] = String(newContext[key]);
        }
        if (required && !(key in context)) {
            display_1.display.warn(`The property ${key} of ${name} context is required; context will not be sent to the intake.`);
        }
    }
    return newContext;
}
function createContextManager(name = '', { customerDataTracker, propertiesConfig = {}, } = {}) {
    let context = {};
    const changeObservable = new observable_1.Observable();
    const contextManager = {
        getContext: () => (0, mergeInto_1.deepClone)(context),
        setContext: (newContext) => {
            if ((0, typeUtils_1.getType)(newContext) === 'object') {
                context = (0, sanitize_1.sanitize)(ensureProperties(newContext, propertiesConfig, name));
                customerDataTracker === null || customerDataTracker === void 0 ? void 0 : customerDataTracker.updateCustomerData(context);
            }
            else {
                contextManager.clearContext();
            }
            changeObservable.notify();
        },
        setContextProperty: (key, property) => {
            context[key] = (0, sanitize_1.sanitize)(ensureProperties({ [key]: property }, propertiesConfig, name)[key]);
            customerDataTracker === null || customerDataTracker === void 0 ? void 0 : customerDataTracker.updateCustomerData(context);
            changeObservable.notify();
        },
        removeContextProperty: (key) => {
            delete context[key];
            customerDataTracker === null || customerDataTracker === void 0 ? void 0 : customerDataTracker.updateCustomerData(context);
            ensureProperties(context, propertiesConfig, name);
            changeObservable.notify();
        },
        clearContext: () => {
            context = {};
            customerDataTracker === null || customerDataTracker === void 0 ? void 0 : customerDataTracker.resetCustomerData();
            changeObservable.notify();
        },
        changeObservable,
    };
    return contextManager;
}
//# sourceMappingURL=contextManager.js.map